//#include <logger.hpp>
#include <simple_logger.hpp>

#ifndef logfilename
#define logfilename "logger.log"
#endif

//To print the sevirity level in the log file
std::ostream& operator<<(std::ostream& out, severityLevel level){
	
	switch(level){
		case unknown: out<<"";
			break;
		case trace: out<<"Trace";
			break;
		case debug: out<<"Debug";
			break;
		case info: out<<"Info";
			break;
		case warning: out<<"Warning";
			break;
		case error: out<<"Error";
			break;
		case fatal: out<<"Fatal";
			break;
	}
	return out;
}

//Create the logger instance (should be thread safe)
logger logger::instance; //Initialise the singleton

//Construct the logger, so every log is written in a text file and every log with a severity over error is printed to the console
//The log in the file are ordered
logger::logger(){
	boost::shared_ptr<boost::log::core> core=boost::log::core::get();
	
	//Add global attributes so every message get a timestamp and the thread's id
	core->add_global_attribute("ThreadID", boost::make_shared<boost::log::attributes::current_thread_id>());
	core->add_global_attribute("TimeStamp", boost::make_shared<boost::log::attributes::local_clock>());

	//Create the log file and put a first line telling when it has been created
	boost::shared_ptr<std::ostream> fileStream(boost::make_shared<std::ofstream>(logfilename));
	if (!fileStream->good())
		throw std::runtime_error("Failed to open the log file");
	boost::posix_time::ptime now=boost::posix_time::second_clock::local_time();
	(*fileStream) << "Log started : "<<boost::posix_time::to_simple_string(now)<<std::endl<<std::endl;	
	
	//Create the file sink which puts log into the log file with a specific format
	fileSink=boost::make_shared<textSink>(boost::make_shared<boost::log::sinks::text_ostream_backend>(), boost::log::keywords::order = boost::log::make_attr_ordering<boost::posix_time::ptime>("TimeStamp", std::less<boost::posix_time::ptime>())
	, boost::log::keywords::ordering_window = boost::posix_time::seconds(1), boost::log::keywords::start_thread=true
	);
	fileSink->locked_backend()->add_stream(fileStream);
	fileSink->locked_backend()->auto_flush(true);

	typedef boost::log::aux::add_common_attributes_constants< char > traits_t;
	fileSink->locked_backend()->set_formatter(
		boost::log::formatters::stream
		<< "[" << boost::log::formatters::date_time< boost::posix_time::ptime >(traits_t::time_stamp_attr_name(), boost::log::keywords::format = "%H:%M:%s")
		<< "] [" << boost::log::formatters::if_ (boost::log::filters::has_attr<std::string>("ThreadName"))
		[
			boost::log::formatters::stream << boost::log::formatters::attr<std::string>("ThreadName")
		]
		.else_
		[
			boost::log::formatters::stream << boost::log::formatters::attr< boost::log::attributes::current_thread_id::held_type >(traits_t::thread_id_attr_name())
		]
		<< "] " << boost::log::formatters::if_ (boost::log::filters::attr<severityLevel>("Severity")!=unknown)
		[
			boost::log::formatters::stream << boost::log::formatters::attr<severityLevel>("Severity") << " : "
		]
		.else_
		[
			boost::log::formatters::stream << " "
		]
		<< boost::log::formatters::message< char >()
	);

	core->add_sink(fileSink);

	//Create the console sink that displays only messages that have a severity greater or equal to error (error ans fatal)
	consoleSink=boost::make_shared<textSink>(boost::make_shared<boost::log::sinks::text_ostream_backend>(), boost::log::keywords::order = boost::log::make_attr_ordering<boost::posix_time::ptime>("TimeStamp", std::less<boost::posix_time::ptime>())
	, boost::log::keywords::ordering_window = boost::posix_time::seconds(1), boost::log::keywords::start_thread=true
	);
	consoleSink->locked_backend()->add_stream(boost::make_shared<std::ofstream>("/dev/stdout")); //Would be better to be able of doing make_shared<std::ostream>(&std::clog) but indice core dumped
	consoleSink->locked_backend()->auto_flush(true);
	consoleSink->set_filter(boost::log::filters::attr<severityLevel>("Severity") >= error);
	core->add_sink(consoleSink);
}

logger::~logger(){
	boost::shared_ptr<boost::log::core> core=boost::log::core::get();
	
	//Append all the pending messages
	core->remove_sink(fileSink);
	core->remove_sink(consoleSink);

	fileSink->stop();
	fileSink->feed_records(boost::posix_time::seconds(0)); //For the oredering asynchronous version
	fileSink.reset();

	consoleSink->stop();
	consoleSink->feed_records(boost::posix_time::seconds(0)); //For the oredering asynchronous version
	consoleSink.reset();
}
